@@ -1,15 +1,21 @@
 # encoding=utf-8
+import datetime
 import itertools
 import sys
 import time
 import tornado.testing
+import toro
+import hashlib
+import zlib
+
 
 from wpull.backport.testing import unittest
 from wpull.util import (to_bytes, sleep, to_str, datetime_str, OrderedDefaultDict,
     wait_future, TimedOut, python_version, filter_pem, detect_encoding,
-    parse_iso8601_str, printable_bytes, DeflateDecompressor)
-import hashlib
-import zlib
+    parse_iso8601_str, printable_bytes, DeflateDecompressor, AdjustableSemaphore)
+
+
+DEFAULT_TIMEOUT = 30
 
 
 class TestUtil(unittest.TestCase):
@@ -205,3 +211,70 @@ class TestUtilAsync(tornado.testing.AsyncTestCase):
             self.assertEqual('uh-oh', error.args[0])
         else:
             self.assertTrue(False)
+
+    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    def test_adjustable_semaphore(self):
+        semaphore = AdjustableSemaphore(value=2)
+
+        yield semaphore.acquire(deadline=datetime.timedelta(seconds=0.1))  # value = 1
+        yield semaphore.acquire(deadline=datetime.timedelta(seconds=0.1))  # value = 2
+
+        try:
+            yield semaphore.acquire(deadline=datetime.timedelta(seconds=0.1))
+        except toro.Timeout:
+            pass
+        else:
+            self.fail()
+
+        semaphore.set_max(3)
+        self.assertEqual(3, semaphore.max)
+
+        yield semaphore.acquire(deadline=datetime.timedelta(seconds=0.1))  # value = 3
+
+        try:
+            yield semaphore.acquire(deadline=datetime.timedelta(seconds=0.1))
+        except toro.Timeout:
+            pass
+        else:
+            self.fail()
+
+        semaphore.set_max(1)
+        self.assertEqual(1, semaphore.max)
+
+        try:
+            yield semaphore.acquire(deadline=datetime.timedelta(seconds=0.1))
+        except toro.Timeout:
+            pass
+        else:
+            self.fail()
+
+        semaphore.release()  # value = 2
+
+        try:
+            yield semaphore.acquire(deadline=datetime.timedelta(seconds=0.1))
+        except toro.Timeout:
+            pass
+        else:
+            self.fail()
+
+        semaphore.release()  # value = 1
+
+        try:
+            yield semaphore.acquire(deadline=datetime.timedelta(seconds=0.1))
+        except toro.Timeout:
+            pass
+        else:
+            self.fail()
+
+        semaphore.release()  # value = 0
+
+        yield semaphore.acquire(deadline=datetime.timedelta(seconds=0.1))
+
+        semaphore.release()
+
+        self.assertRaises(ValueError, semaphore.release)
+
+        def set_neg_max():
+            semaphore.set_max(-1)
+
+        self.assertRaises(ValueError, set_neg_max)
